@@ -35,8 +35,6 @@ class ConstantString(Enum):
     AWS_SECRET_ACCESS_KEY = "aws_secret_access_key"
 
 
-
-
 ###### ---------- Global parameters ---------- ######
 
 
@@ -61,15 +59,15 @@ class DynamoDBGlobalConfiguration:
 
     @classmethod
     def get_instance(
-        cls,
-        ddb: object = None,
-        tables: dict = None,
-        use_aws_cli_credentials: bool = True,
-        access_params: dict = None,
-        single_table_name_env_var: str = "DYNAMODB_TABLE_NAME",
-        single_table_pk_attr_name: str = "pk",
-        single_table_sk_attr_name: str = "sk",
-        single_table_inverse_index_properties: dict = None,
+            cls,
+            ddb: object = None,
+            tables: dict = None,
+            use_aws_cli_credentials: bool = True,
+            access_params: dict = None,
+            single_table_name_env_var: str = "DYNAMODB_TABLE_NAME",
+            single_table_pk_attr_name: str = "pk",
+            single_table_sk_attr_name: str = "sk",
+            single_table_inverse_index_properties: dict = None,
             overwrite_existing_instance: bool = False,
             log_debug_messages: bool = True
     ):
@@ -115,7 +113,6 @@ class DynamoDBGlobalConfiguration:
                 (The default will be changed to False in the near future)
         """
 
-
         if cls._instance is None or overwrite_existing_instance:
             tables = tables if tables is not None else {}
             if single_table_inverse_index_properties is None:
@@ -138,8 +135,9 @@ class DynamoDBGlobalConfiguration:
                          ConstantString.AWS_SECRET_ACCESS_KEY.value
                      } <= set(access_params):
                 use_aws_cli_credentials = True
-                logging.warning("The provided access_params " + str(access_params) + " is missing required values, trying"
-                                                                                   " to use the aws cli credentials")
+                logging.warning(
+                    "The provided access_params " + str(access_params) + " is missing required values, trying"
+                                                                         " to use the aws cli credentials")
             cls._instance = cls(
                 ddb,
                 tables,
@@ -158,7 +156,7 @@ class DynamoDBGlobalConfiguration:
         return cls._instance is not None
 
     def get_connection(self):
-        if self._ddb is None: # Connect to the database if ddb is None
+        if self._ddb is None:  # Connect to the database if ddb is None
             try:
                 if self.use_aws_cli_credentials:
                     self._ddb = boto3.resource("dynamodb")
@@ -206,11 +204,11 @@ def _list_if_none(a_list):
 
 
 def _wrap_class(  # noqa: C901
-    cls=None,  # noqa: C901
-    table_name: str = None, # noqa: C901
-    unique_id: str = None,  # noqa: C901
-    mapping_schema: dict = None,  # noqa: C901
-    ignore_attributes: list = None,  # noqa: C901
+        cls=None,  # noqa: C901
+        table_name: str = None,  # noqa: C901
+        unique_id: str = None,  # noqa: C901
+        mapping_schema: dict = None,  # noqa: C901
+        ignore_attributes: list = None,  # noqa: C901
 ):  # noqa: C901
     """
     Adds classmethods to the provided class, to be used by only the dynamo_entity decorator
@@ -223,7 +221,6 @@ def _wrap_class(  # noqa: C901
     # May causes collisions if there are other entities with the same name
     table_name = table_name if table_name is not None else cls.__name__
 
-
     # The methods to be added to the class
     @classmethod
     def dynamo_table_name(cls):
@@ -259,11 +256,11 @@ def _wrap_class(  # noqa: C901
 
 
 def dynamo_entity(
-    cls=None,
-    table_name: str = None,
-    unique_id: str = None,
-    mapping_schema: dict = None,
-    ignore_attributes: Union[list, tuple] = None,
+        cls=None,
+        table_name: str = None,
+        unique_id: str = None,
+        mapping_schema: dict = None,
+        ignore_attributes: Union[list, tuple] = None,
 ):
     """
     Wraps a class so it is mappable to DynamoDB. Use it as a decorator.
@@ -507,9 +504,9 @@ class DynamoCrudRepository(CrudRepository, metaclass=abc.ABCMeta):
     map_filled: bool = False
 
     def __init__(
-        self,
-        entity_type: T,
-        dynamo_table_name: str = None
+            self,
+            entity_type: T,
+            dynamo_table_name: str = None
     ):
         """
         Creates a new DynamoCrudRepository for a specific dynamo_entity class
@@ -559,7 +556,7 @@ class DynamoCrudRepository(CrudRepository, metaclass=abc.ABCMeta):
             except Exception as ex:
                 # LOG
                 logging.warning("Not able to query table " + str(self.table_name) +
-                              " with key_condition_expression" + str(key_condition_expression))
+                                " with key_condition_expression" + str(key_condition_expression))
                 return None
         else:
             try:
@@ -570,8 +567,8 @@ class DynamoCrudRepository(CrudRepository, metaclass=abc.ABCMeta):
             except Exception as ex:
                 # LOG
                 logging.warning("Not able to query table " + str(self.table_name) +
-                              " with index_name " + str(index_name) +
-                              "and key_condition_expression" + str(key_condition_expression))
+                                " with index_name " + str(index_name) +
+                                "and key_condition_expression" + str(key_condition_expression))
 
         if "Items" in response:
             items = response["Items"]
@@ -673,9 +670,11 @@ class DynamoCrudRepository(CrudRepository, metaclass=abc.ABCMeta):
         """
 
         if not isclass(instance_attr_type):  # Try to get the class type
-            if hasattr(instance_attr_type,  "__supertype__"):  # Try to get the class type from typings.NewType
+            # Try to get the class type from typings.NewType
+            if hasattr(instance_attr_type, "__supertype__"):
                 instance_attr_type = instance_attr_type.__supertype__
-            elif hasattr(instance_attr_type,  "__origin__"):  # Try to get the class type from typings.Generic
+            # Try to get the class type from typings.Generic. Not an elif, the NewType may be an alias for a Generic
+            if hasattr(instance_attr_type, "__origin__"):
                 instance_attr_type = instance_attr_type.__origin__
 
         if ConstantString.ITEM2OBJ_CONV.value in self.map_dict[instance_attr_name]:
@@ -773,7 +772,7 @@ class DynamoCrudRepository(CrudRepository, metaclass=abc.ABCMeta):
             "method": "SingleTableCrudRepository.find_by_keys",
             "Error": "No error, the method was just called",
             "Provided_Args": {
-                            "keys": str(keys),
+                "keys": str(keys),
             }
         }
         if isinstance(keys, list):  # Produce dict from list
@@ -903,6 +902,9 @@ class DynamoCrudRepository(CrudRepository, metaclass=abc.ABCMeta):
                 )
         elif self.map_dict[instance_attr_name][ConstantString.ITEM_ATTR_TYPE.value] == "BOOL":  # case 'BOOL' (boolean)
             item_attr_val = 1 if instance_attr_val else 0
+        elif self.map_dict[instance_attr_name][ConstantString.ITEM_ATTR_TYPE.value] in {"BS", "NS", "SS"}:
+
+            item_attr_val = set(instance_attr_val)
         else:  # default (string)
             # Consider special cases and use specific string formats
             # datetime
@@ -946,9 +948,9 @@ class DynamoCrudRepository(CrudRepository, metaclass=abc.ABCMeta):
         # Get every attribute of obj, ignoring private members and methods
         for instance_attr in getmembers(obj):
             if (
-                (not instance_attr[0].startswith("_"))  # ignore private attributes
-                and (not ismethod(instance_attr[1]))  # ignore methods
-                and (not instance_attr[0] in obj._dynamo_ignore())  # ignore the attributes on the list to ignore
+                    (not instance_attr[0].startswith("_"))  # ignore private attributes
+                    and (not ismethod(instance_attr[1]))  # ignore methods
+                    and (not instance_attr[0] in obj._dynamo_ignore())  # ignore the attributes on the list to ignore
             ):
                 instance_attr_name = instance_attr[0]
                 instance_attr_val = instance_attr[1]
@@ -1025,7 +1027,7 @@ class DynamoCrudRepository(CrudRepository, metaclass=abc.ABCMeta):
         Removes every entity(ies) that matches the provided keys
 
         Args:
-            keys(Union[Iterable[dict], Iterable[list]]):
+            keys_list(Union[Iterable[dict], Iterable[list]]):
                 a (pair of) key(s) that identify the entity(ies)
 
         Returns:
@@ -1145,10 +1147,22 @@ class DynamoCrudRepository(CrudRepository, metaclass=abc.ABCMeta):
             if hasattr(python_type, "__supertype__"):  # Try to get the class type from typings.NewType
                 python_type = python_type.__supertype__
             elif hasattr(python_type, "__origin__"):  # Try to get the class type from typings.Generic
-                python_type = python_type.__origin__
+                # This will convert compatible python sets to DynamoDB sets
+                if (hasattr(python_type, "__args__") and issubclass(python_type.__origin__, (set, frozenset))
+                        and issubclass(python_type.__args__[0], (bytes, bytearray, str, int, float, Decimal))):
+                    # python_type.__args__ is a tuple that holds the typehints
+                    if issubclass(python_type.__args__[0], (bytes, bytearray)):
+                        return "BS"
+                    if issubclass(python_type.__args__[0], str):
+                        return "SS"
+                    if issubclass(python_type.__args__[0], (int, float, Decimal)):
+                        return "NS"
+
+                else:
+                    python_type = python_type.__origin__
         # if using a specific library like numpy or pandas, the user should specify the "N" type himself
         if issubclass(python_type, (int, float, Decimal)):
-            dynamo_type = "N"
+            return "N"
         elif issubclass(
                 python_type,
                 (
@@ -1165,20 +1179,19 @@ class DynamoCrudRepository(CrudRepository, metaclass=abc.ABCMeta):
                         UUID,
                 ),
         ):
-            dynamo_type = "S"
+            return "S"
         elif issubclass(python_type, bool):
-            dynamo_type = "BOOL"
+            return "BOOL"
         elif issubclass(
                 python_type, (bytes, bytearray, object)
         ):  # general objects will be pickled
-            dynamo_type = "B"
+            return "B"
         else:  # this will probably never be reached since general objects are converted to bytes
-            dynamo_type = "S"
-        return dynamo_type
+            return "S"
 
     def _fill_map_dict(self, cls):
         """
-        Fill the entity_tupe map
+        Fill the entity_type map
         Args:
             cls: the mapped class
         """
@@ -1199,6 +1212,7 @@ class DynamoCrudRepository(CrudRepository, metaclass=abc.ABCMeta):
 
     ###### ---------- Repository Interfaces and Implementation ---------- ######
 
+
 class SingleTableDynamoCrudRepository(DynamoCrudRepository):
     _logical_table_name: str = None
 
@@ -1217,15 +1231,12 @@ class SingleTableDynamoCrudRepository(DynamoCrudRepository):
                 The name of the real table on DynamoDB
         """
         if dynamo_table_name is None:
-            if hasattr(entity_type, "_dynamo_table_name") and entity_type._dynamo_table_name() is not None:
-                dynamo_table_name = entity_type._dynamo_table_name()
+            import os
+            global_values = DynamoDBGlobalConfiguration.get_instance()
+            if global_values.single_table_name_env_var in os.environ:
+                dynamo_table_name = os.environ[global_values.single_table_name_env_var]
             else:
-                import os
-                global_values = DynamoDBGlobalConfiguration.get_instance()
-                if global_values.single_table_name_env_var in os.environ:
-                    dynamo_table_name = os.environ[global_values.single_table_name_env_var]
-                else:
-                    raise ValueError("You need to provide the dynamo table name")
+                raise ValueError("You need to provide the dynamo table name")
         self._logical_table_name = entity_type._dynamo_table_name()
         super().__init__(entity_type, dynamo_table_name)
 
@@ -1280,7 +1291,7 @@ class SingleTableDynamoCrudRepository(DynamoCrudRepository):
         inverse_index = DynamoDBGlobalConfiguration.get_instance().single_table_inverse_index_properties
         index_name = inverse_index[ConstantString.INVERSE_INDEX_NAME.value]
         key_condition_expression = Key(inverse_index[ConstantString.INVERSE_INDEX_PK.value]).eq(
-                    self.entity_type._dynamo_table_name())
+            self.entity_type._dynamo_table_name())
         _, count = self.find_collection(key_condition_expression, index_name)
         return count if count is not None else 0
 
@@ -1351,7 +1362,7 @@ class SingleTableDynamoCrudRepository(DynamoCrudRepository):
         inverse_index = DynamoDBGlobalConfiguration.get_instance().single_table_inverse_index_properties
         index_name = inverse_index[ConstantString.INVERSE_INDEX_NAME.value]
         key_condition_expression = Key(inverse_index[ConstantString.INVERSE_INDEX_PK.value]).eq(
-                    self.entity_type._dynamo_table_name())
+            self.entity_type._dynamo_table_name())
         entity_list, _ = self.find_collection(key_condition_expression, index_name)
         return entity_list
 
@@ -1469,7 +1480,7 @@ class MultiTableDynamoCrudRepository(DynamoCrudRepository):
         inverse_index = DynamoDBGlobalConfiguration.get_instance().single_table_inverse_index_properties
         index_name = inverse_index[ConstantString.INVERSE_INDEX_NAME.value]
         key_condition_expression = Key(inverse_index[ConstantString.INVERSE_INDEX_PK.value]).eq(
-                    self.entity_type._dynamo_table_name())
+            self.entity_type._dynamo_table_name())
         _, count = self.find_collection(key_condition_expression, index_name)
         return count if count is not None else 0
 
@@ -1547,7 +1558,7 @@ class MultiTableDynamoCrudRepository(DynamoCrudRepository):
         inverse_index = DynamoDBGlobalConfiguration.get_instance().single_table_inverse_index_properties
         index_name = inverse_index[ConstantString.INVERSE_INDEX_NAME.value]
         key_condition_expression = Key(inverse_index[ConstantString.INVERSE_INDEX_PK.value]).eq(
-                    self.entity_type._dynamo_table_name())
+            self.entity_type._dynamo_table_name())
         entity_list, _ = self.find_collection(key_condition_expression, index_name)
         return entity_list
 
